import java.util.concurrent.Callable;

public class RetryUtility {

    public static <T> T retry(Callable<T> task, int maxRetries, long initialDelayMillis) throws Exception {
        int attempt = 0;
        while (true) {
            try {
                return task.call();  // Try to execute the task
            } catch (Exception e) {
                attempt++;
                if (attempt > maxRetries) {
                    throw new Exception("All retry attempts failed", e);
                }

                long backoffTime = initialDelayMillis * (1L << (attempt - 1));  // Exponential backoff
                System.out.println("Attempt " + attempt + " failed: " + e.getMessage());
                System.out.println("Retrying in " + backoffTime + "ms...");

                Thread.sleep(backoffTime);
            }
        }
    }

    public static void main(String[] args) {
        try {
            String result = retry(() -> {
                // Simulate network call that fails randomly
                if (Math.random() < 0.7) {
                    throw new RuntimeException("Network error!");
                }
                return "Success!";
            }, 5, 1000);  // Retry up to 5 times, starting with 1 second delay

            System.out.println("Final Result: " + result);
        } catch (Exception e) {
            System.err.println("Operation failed after retries: " + e.getMessage());
        }
    }
}
